import EventEmitter from 'events';
import { CoreClient } from './client.js';
import { get_config } from '../config.js';
import { check_process, spawn_process } from './cmd.js';
import { ensure_file_exists, ensure_path_exists } from './util.js';
const RAND_PORT = () => Math.floor((Math.random() * 10 ** 5 % 25_000) + 25_000);
export class CoreDaemon extends EventEmitter {
    constructor(config) {
        super();
        const opt = get_config(config);
        const { isolated, throws } = opt;
        if (isolated) {
            const port = RAND_PORT();
            opt.peer_port = port;
            opt.rpc_port = port + 1;
        }
        this._client = new CoreClient(opt);
        this._closing = false;
        this.params = [
            `-chain=${opt.network}`,
            `-port=${opt.peer_port}`,
            `-rpcport=${opt.rpc_port}`,
            ...opt.params,
            ...opt.core_params
        ];
        if (opt.confpath !== undefined) {
            this.params.push(`-conf=${opt.confpath}`);
        }
        if (opt.datapath !== undefined) {
            this.params.push(`-datadir=${opt.datapath}`);
        }
        process.on('uncaughtException', async (err) => {
            if (!this._closing) {
                console.log('[core] Daemon caught an error, exiting...');
                await this.shutdown();
                this._closing = true;
            }
            if (throws) {
                throw err;
            }
            else {
                console.log(err.message);
            }
        });
        process.on('unhandledRejection', async (reason) => {
            if (!this._closing) {
                console.log('[core] Daemon caught a promise rejection, exiting...');
                await this.shutdown();
                this._closing = true;
            }
            const msg = String(reason);
            if (throws) {
                throw new Error(msg);
            }
            else {
                console.log(msg);
            }
        });
        this._opt = opt;
    }
    get client() {
        return this._client;
    }
    get opt() {
        return this._opt;
    }
    on(event, cb) {
        return super.on(event, cb);
    }
    once(event, cb) {
        return super.once(event, cb);
    }
    emit(event, arg) {
        return super.emit(event, arg);
    }
    async _start(params = []) {
        const { confpath, corepath, datapath, debug, throws, timeout } = this.opt;
        if (confpath !== undefined) {
            await ensure_file_exists(confpath);
        }
        if (datapath !== undefined) {
            await ensure_path_exists(datapath);
        }
        const exec = corepath ?? 'bitcoind';
        const msg = 'loadblk thread exit';
        params = [...this.params, ...params];
        if (debug) {
            console.log('[core] exec :', exec);
            console.log('[core] data :', datapath);
            console.log('[core] args :', params.join(' '));
        }
        this._proc = await spawn_process(exec, params, msg, throws, timeout);
    }
    async startup(params = []) {
        const { isolated, verbose } = this.opt;
        if (isolated) {
            await this._start(params);
        }
        else {
            if (await check_process('bitcoin-qt')) {
                if (verbose)
                    console.log('[core]: Using existing bitcoin QT instance...');
            }
            else if (await check_process('bitcoind')) {
                if (verbose)
                    console.log('[core] Using existing bitcoin daemon...');
            }
            else {
                if (verbose)
                    console.log('[core] Starting new bitcoin daemon...');
                await this._start(params);
            }
        }
        this.emit('ready', this.client);
        return this.client;
    }
    async shutdown() {
        if (this._proc !== undefined) {
            const is_dead = this._proc.kill();
            if (!is_dead)
                this._proc.kill('SIGKILL');
        }
    }
    async run(...methods) {
        if (this._proc === undefined) {
            await this.startup();
        }
        const jobs = methods.map(async (fn) => {
            return Promise.resolve(fn(this.client))
                .then(() => null)
                .catch((err) => err);
        });
        const ret = await Promise.all(jobs);
        await this.shutdown();
        return ret;
    }
}
//# sourceMappingURL=core.js.map